@@ -1,4 +1,4 @@
-/* Copyright (C) 2006-2007 G.P. Halkes
+/* Copyright (C) 2006-2008 G.P. Halkes
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 3, as
    published by the Free Software Foundation.
@@ -20,11 +20,16 @@
 #include <stdlib.h>
 #include <limits.h>
 
+/* Make sure that if gettext is not used, the _ macro does exist */
+#ifndef _
+#define _(_x) (_x)
+#endif
+
 /** Define an option parsing function.
 	@param name The name of the function to define.
 */
 #define PARSE_FUNCTION(name) void name(int argc, char **argv) {\
-	char *optarg; \
+	char *optArg; \
 	int optargind;
 
 /** Indicate the start of option processing.
@@ -34,32 +39,32 @@
 */
 #define OPTIONS \
 	for (optargind = 1; optargind < argc; optargind++) { \
-		int optind; \
-		optind = optargind; \
+		int optInd; \
+		optInd = optargind; \
 		if (argv[optargind][0] == '-') { \
 			size_t optlength; \
 			ArgType opttype; \
 \
 			if (argv[optargind][1] == '-') { \
-				if ((optarg = strchr(argv[optargind], '=')) == NULL) { \
+				if ((optArg = strchr(argv[optargind], '=')) == NULL) { \
 					optlength = strlen(argv[optargind]); \
 				} else { \
-					optlength = optarg - argv[optargind]; \
-					optarg++; \
-					if (*optarg == 0) { \
-						fatal(_("Option %.*s has zero length argument\n"), (int) optlength, argv[optind]); \
+					optlength = optArg - argv[optargind]; \
+					optArg++; \
+					if (*optArg == 0) { \
+						fatal(_("Option %.*s has zero length argument\n"), (int) optlength, argv[optInd]); \
 					} \
 				} \
 				opttype = LONG; \
 			} else { \
 				optlength = 2; \
 				if (argv[optargind][1] != 0 && argv[optargind][2] != 0) \
-					optarg = argv[optargind] + 2; \
+					optArg = argv[optargind] + 2; \
 				else \
-					optarg = NULL; \
+					optArg = NULL; \
 				opttype = SHORT; \
 			} \
-			if (optlength > INT_MAX) optlength = INT_MAX; 
+			if (optlength > INT_MAX) optlength = INT_MAX;
 /* The last line above is to make sure the cast to int in error messages does
    not overflow. */
 
@@ -74,26 +79,26 @@
 
 /** The string containing the current option, including integrated option
 		argument, if applicable.
-	
+
 	To print the option, use the %.*s format specifier, and include
 	optlength, CURRENT_OPTION in the arguments of printf.
 */
-#define CURRENT_OPTION argv[optind]
+#define CURRENT_OPTION argv[optInd]
 
 /** Internal macro to check whether the requirements regarding option arguments
 		have been met. */
 #define CHECK_ARG(argReq) \
 	switch(argReq) { \
 		case NO_ARG: \
-			if (optarg != NULL) { \
-				fatal(_("Option %.*s does not take an argument\n"), (int) optlength, argv[optind]); \
+			if (optArg != NULL) { \
+				fatal(_("Option %.*s does not take an argument\n"), (int) optlength, argv[optInd]); \
 			} \
 			break; \
 		case REQUIRED_ARG: \
-			if (optarg == NULL && (optargind+1 >= argc)) { \
-				fatal(_("Option %.*s requires an argument\n"), (int) optlength, argv[optind]); \
+			if (optArg == NULL && (optargind+1 >= argc)) { \
+				fatal(_("Option %.*s requires an argument\n"), (int) optlength, argv[optInd]); \
 			} \
-			if (optarg == NULL) optarg = argv[++optargind]; \
+			if (optArg == NULL) optArg = argv[++optargind]; \
 			break; \
 		default: \
 			break; \
@@ -167,12 +172,12 @@
 	long value; \
 	errno = 0; \
 	\
-	value = strtol(optarg, &endptr, 10); \
+	value = strtol(optArg, &endptr, 10); \
 	if (*endptr != 0) { \
-		fatal("Garbage after value for %.*s option\n", (int) optlength, argv[optind]); \
+		fatal(_("Garbage after value for %.*s option\n"), (int) optlength, argv[optInd]); \
 	} \
 	if (errno != 0 || value < min || value > max) { \
-		fatal("Value for %.*s option (%ld) is out of range\n", (int) optlength, argv[optind], value); \
+		fatal(_("Value for %.*s option (%ld) is out of range\n"), (int) optlength, argv[optInd], value); \
 	} \
 	var = (int) value; } while(0)
 
